# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


# This package contains base functions which can are used into wosedon 
# applications. For example base parser for config, model dir and total 
# number of iterations. There is a general method to convert a whole 
# rank to string representation, to adds annotations to CCL files and 
# returns its. An output of annotated CCLs may be returned to stdout or 
# to given filename.

'''
PoSeDoN accept different models for different part of speech. 
It is possible to disambiguate the different POS using different models 
and also graphs.
'''

import sys
import corpus2
import run_wosedon
import wosedon_base
import wosedon_plugin
from context import context as ccc

try:
    import argcomplete
except ImportError:
    argcomplete = None

def make_parser(desc):
  """!
  Makes the parser for comman line options.
  @param desc An application descriptions
  @type desc: String
  @return Argument parser
  @rtype: argparse.ArgumentParser
  """
  aparser = wosedon_base.make_parser(desc)
  aparser.add_argument('-f', '--cclfile', dest='ccl',
                       required=True, type=str, help='Ccl file.')
  aparser.add_argument('-r', '--relcclfile', dest='relccl',
                       required=False, type=str, help='Relccl file')
  aparser.add_argument('-o', '--out-file', dest='out_file', default='-',
                       help=(
                         'Output file. Default is set to write output '
                         'to stdout (\"-\") if set filename, then output will be ' 
                         'stored to given filename.'))
  aparser.add_argument('-b', '--batch', dest='batch',
                       action='store_true', help=(
                         'If batch mode is enabled, then into -f file is '
                         'the index of files to be disambiguated. The index '
                         'contains the paths to the files: one path per one '
                         'line. New, disambiguated file has new name with '
                         'postfix like a *.wosedon.xml')),

  aparser.add_argument('-Other', '--other-pos-cfg', dest='other_pos_cfg', 
                       required=True, help=(
                         'This config is used when the specific POS has not '
                         'assigned config. This option is required.'))
  aparser.add_argument('-Noun', '--noun-pos-cfg', dest='noun_pos_cfg', 
                       required=False, help=(
                         'This config is used while the Noun words will be '
                         'disambiguated. When is not assigned, '
                         'then the config from option -O will be used.'))
  aparser.add_argument('-Verb', '--verb-pos-cfg', dest='verb_pos_cfg', 
                       required=False, help=(
                         'This config is used while the Verb words will be '
                         'disambiguated. When is not assigned, '
                         'then the config from option -O will be used.'))

  aparser.add_argument('-vd', '--visualisation-dir', dest='visualisation_dir',
                       required=False, help=(
                         'Path to directory for PageRank visualisation.'))
  return aparser

def load_models(oconfig, nconfig, vconfig, mdir, add_args, verbose):
  """!
  For the specific config, loads the appropriate models
  
  @param oconfig Config path to other POS
  @type oconfig: String
  @param nconfig Config path to noun POS
  @type nconfig: String
  @param vconfig Config path to verb POS
  @type vconfig: String
  @param add_args Additional options (derived from wosedon_base)
  @type: list
  @param verbose If set to True then PoSeDon is more verbose
  @verbose type: Boolean
  @return dictionary where as the key is used POS and as value the model
  @rtype: dict[pos_int]=>WoSeDonPlugin
  """
  mdl_dict = {}
  mdl_dict['o'] = wosedon_plugin.WoSeDonPlugin(
    oconfig, mdir, verbose=verbose, argv=add_args)

  if nconfig:
    mdl_dict['n'] = wosedon_plugin.WoSeDonPlugin(
      nconfig, mdir, verbose=verbose, argv=add_args)
  #else:
  #  mdl_dict['n'] = mdl_dict['o']

  if vconfig:
    mdl_dict['v'] = wosedon_plugin.WoSeDonPlugin(
      vconfig, mdir, verbose=verbose, argv=add_args)
  #else:
  #  mdl_dict['v'] = mdl_dict['o']

  return mdl_dict

'''
def add_annotation(models, rankings, out_cclfilepath):
  full_pos_mask = None
  available_models = models.keys()
  for (token, tagset), node_rank in rankings['o'].iteritems():
    if not full_pos_mask:
      full_pos_mask = corpus2.get_attribute_mask(tagset, '')
    if not node_rank or not token or not len(token.orth_utf8()):
      continue

    print len(token.lexemes())
    tag = token.get_preferred_lexeme(tagset)#.tag()
    # pos_mask = tag.get_masked(full_pos_mask)
    print ('Added annotation for token %s' % token.orth_utf8())

    # ref_pos = wsdctx.get_token_coarse_pos(token.clone())
    # refmodel = models[ref_pos] if ref_pos in available_models else models['o']
'''
def add_annotation(models, rankdict, out_cclfilepath):
  tagset = models['o'].config.resources().tagset()
  full_pos_mask = corpus2.get_attribute_mask(tagset, '')
  ctx  = ccc.WSDContext(tagset, [])

  for i, rctx in enumerate(rankdict['o']):
    for j, (token, node_rank) in enumerate(rctx):
      if not node_rank:
        continue

      md_ptr = token.get_metadata()
      if not md_ptr:
        token.create_metadata()
        md_ptr = token.get_metadata()

      # Coarse POS for token.
      tag = token.get_preferred_lexeme(tagset).tag()
      pos_mask = tag.get_masked(full_pos_mask)
      pos_str = ctx.convert_to_coarse_pos(
        tagset.tag_to_symbol_string(pos_mask))

      synset_rank = None
      fst_rank_synid = None
      fst_unitsstr = None
      if pos_str in rankdict:
        model = models[pos_str]
        node_rank = rankdict[pos_str][i][j][1] # w tym miejscu moze nie dzialac jesli zostanie ustawiony parametr alpha
        synset_rank = [(model.graph.get_node_attribute("synset", r[0]).synset_id, r[1]) 
                       for r in node_rank]
        # best synset ID
        fst_rank_synid = str(synset_rank[0][0])
        # best unit string
        fst_unitsstr = wosedon_base.make_unitstr(model.graph, node_rank[0][0])
      else:
        model = models['o']
        synset_rank = [(model.graph.get_node_attribute("synset", r[0]).synset_id, r[1]) 
                       for r in node_rank]
        # best synset ID
        fst_rank_synid = str(synset_rank[0][0])
        # best unit string
        fst_unitsstr = wosedon_base.make_unitstr(model.graph, node_rank[0][0])

      whole_rank_norm_str = wosedon_base.make_whole_rank_str(synset_rank)
      md_ptr.set_attribute('sense:ukb:syns_id', fst_rank_synid)
      md_ptr.set_attribute('sense:ukb:syns_rank', whole_rank_norm_str)
      md_ptr.set_attribute('sense:ukb:unitsstr', fst_unitsstr[:-1])

def run_disambiguation(models, args, in_cclfilepath, out_cclfilepath = None):
  """!
  @return dictionary with ranking for each model
  @rtype: List of tuples (token, [ranking, for, token])
  """

  rankdict = {}
  contextdict = {}
  for mkey, model in models.iteritems():
    rankings = []
    # make contextdict
    contextdict[mkey] = model.config.context(in_cclfilepath, in_cclfilepath)

    # run process
    run_wosedon._run_disambiguation(
      model.graph, 
      contextdict[mkey], 
      model.algorithm, 
      model.config,
      args.alpha,
      args.verbose,
      args.visualisation_dir,
      output_filename=None, 
      show_iterations=False,
      model_result_list=rankings)

    '''
    # convert to dictionary: token -> ranking
    rankdict[mkey] = dict()
    for rctx in rankings:
      for t, rank in rctx:
        rankdict[mkey][t, tagset] = rank
    '''
    rankdict[mkey] = rankings

  if out_cclfilepath:
    add_annotation(models, rankdict, out_cclfilepath)
    wosedon_base.write_ccl_file(models['o'].config, 
                                contextdict['o'].ccl_document(), 
                                out_cclfilepath)

  return rankdict


def main(argv=None):
  p = make_parser(__doc__)
  args = p.parse_args(argv)

  # additional arguments
  add_args = []
  if args.iterations:
    add_args.extend(['-it', args.iterations])
  if args.alpha:
    add_args.extend(['-a', args.alpha])

  # load the models
  modeldict = load_models(
    args.other_pos_cfg, args.noun_pos_cfg, args.verb_pos_cfg, 
    args.model_dir, add_args, args.verbose)

  if args.batch:
    with open(args.ccl, 'rt') as batch_file:
      for line in batch_file:
        ccl_filename = line.strip()
        out_ccl_filename = ccl_filename + ".wosedon.xml"
        if args.verbose:
          print >> sys.stderr, 'Running WoSeDoN for file:', ccl_filename
        # run disambiguation and add the annotations to text
        rankdict = run_disambiguation(modeldict, args, ccl_filename, out_ccl_filename)
  else:
    # run disambiguation and add the annotations to text
    rankdict = run_disambiguation(modeldict, args, args.ccl, args.out_file)


if __name__ == '__main__':
  main()


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from wosedon.lesk_filters.leskfilterinterface import LeskFilterInterface 
from wosedon.utils.leskfilteroptions import LeskFilterOptions


MODULE_LESK_FUNCTIONS = 'wosedon.lesk_functions'
MODULE_LESK_FILTERS = 'wosedon.lesk_filters'



class AlgorithmOptions:
  def __init__(self):
    self._damping_factor = 0.85
    self._max_iter = 15
    self._ini_nodes = None
    
    # function to be used by Lesk
    self._lesk_function = None
    # filter to be used by Lesk
    self._lesk_filter = None
    # LeskFilterOptions object
    self._lesk_filter_opts = LeskFilterOptions()

  def set_damping_factor(self, df):
    self._damping_factor = df

  def set_max_iter(self, mi):
    self._max_iter = mi

  def set_ini_nodes(self, inodes):
    self._ini_nodes = inodes
    
  def set_lesk_function(self, func):
    self._lesk_function = func
    
  def set_lesk_filter(self, filtr):
    self._lesk_filter = filtr

  def damping_factor(self):
    return float(self._damping_factor)

  def max_iter(self):
    return int(self._max_iter)

  def ini_nodes(self):
    return self._ini_nodes

  def lesk_filter_opts(self):
    return self._lesk_filter_opts

  def lesk_function(self, resources, graph):
    if isinstance(self._lesk_function, str):
      module = __import__(MODULE_LESK_FUNCTIONS, fromlist=[self._lesk_function])
      self._lesk_function = getattr(module, self._lesk_function)(resources, self.lesk_filter(resources.tagset()), graph, self)
    return self._lesk_function

  def lesk_filter(self, tagset):
    if not isinstance(self._lesk_filter, LeskFilterInterface):
      if not self._lesk_filter:
        self._lesk_filter = 'Yes'
        
      module = __import__(MODULE_LESK_FILTERS, fromlist=[self._lesk_filter])
      self._lesk_filter = getattr(module, self._lesk_filter)(tagset, self._lesk_filter_opts)
    
    return self._lesk_filter












# -*- coding: utf-8 -*-

import unittest
from wosedon.lesk_functions.intersection import Intersection
from wosedon.utils.config import Config






class TestLeskIntersection(unittest.TestCase):

  def setUp(self):
    cfg = Config()
    cfg.parse('tests/#Data/wosedon_intersection.ini')
    
    self.graph = cfg.gbuilders()[0].build_graph()
    self.context = cfg.context('tests/#Data/wiki_ccl.xml', 'tests/#Data/wiki_ccl.xml')
    self.intersection = cfg.algorithm_options().lesk_function(cfg.resources(), self.graph)
  
  
  def test_intersection(self):
    
    definition_data = {
      ('aaaa', 'adv') : 1,
      ('bbbb', 'subst') : 2,
      ('ccccc', 'fin') : 3,
      ('ddd', 'adja') : 4,
      ('eee', 'depr') : 5,
      ('fff', 'inf') : 6,
      ('ggg', 'pact') : 7
    }
    
    context_data = {
      ('aaaa', 'adv') : 4,
      ('bbbb', 'subst') : 2,
      ('ccccc', 'fin') : 4,
      ('ddd', 'adja') : 5,
      ('eee', 'depr') : 6,
      ('fff', 'inf') : 2,
      ('ggg', 'pact') : 1,
      ('afsadfs', 'inf') : 1352352,
      ('hthhj', 'inf') : 356456464,
      ('ewrwre', 'inf') : 2352343
    }
    
    self.assertEquals(self.intersection.compare(definition_data, context_data), 7)
  
  
  def test_run_prepare_node(self):
    self.graph.create_edge_attribute('weight', 'float')
    for node in self.graph.all_nodes():
      res = self.intersection.prepare_node(node)
      self.assertIsInstance(res, dict)
      for k in res:
        self.assertIsInstance(k, tuple)
        self.assertIsInstance(k[0], str)
        self.assertIsInstance(k[1], str)
        self.assertIsInstance(res[k], float)
  
  
  def test_run_prepare_context(self):
    sen = self.context.next()
    while sen:
      res = self.intersection.prepare_context(sen)
      self.assertIsInstance(res, dict)
      for k in res:
        self.assertIsInstance(k, tuple)
        self.assertIsInstance(k[0], str)
        self.assertIsInstance(k[1], str)
        self.assertIsInstance(res[k], float)
      sen = self.context.next()












# -*- coding: utf-8 -*-
import unittest
from wosedon.utils.config import Config

from wosedon.gbuilders.graphbuilderinterface import GraphBuilderInterface
from wosedon.mergers.mergerinterface import MergerInterface
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.reranks.rerankinterface import RerankInterface

from wosedon.gbuilders import SynsetGraphBuilder, SUMOGraphBuilder, LexicalUnitGraphBuilder
from wosedon.mergers import SynsetsSUMOMerger, SynsetsLUMerger2
from wosedon.algorithms import LeskAlg, GTStaticPR, GTPersonalizedPRNorm2
from wosedon.reranks import LemmaRankingNormalizer, LemmaRankingSelecter




class TestConfig(unittest.TestCase):
  
  def setUp(self):
    self.cfg = Config()
  
  
  def test_buid_dict(self):
    self.cfg.parse('tests/#Data/wosedon_dicts.ini')
    a = self.cfg._build_dict('aaa', 'aa', str, int)
    b = self.cfg._build_dict('bbb', 'bb', int, float)
    c = self.cfg._build_dict('ccc', 'cc', str, str)
    
    self.assertEquals(a, {'a':1, 'b':2, 'c':3})
    self.assertEquals(b, {1:-1.1, 2:2.2, -3:-3.3})
    self.assertEquals(c, {'aaa':'bbb', 'zażółć':'gęślą', 'jaźń':'ccc'})
  
  
  def test_weights1(self):
    self.cfg.parse('tests/#Data/wosedon.ini')
    self.assertEquals(self.cfg.weights(), None)
  
  
  def test_weights2(self):
    self.cfg.parse('tests/#Data/wosedon_lu.ini')
    self.assertIsInstance(self.cfg.weights(), dict)
    self.assertEquals(len(self.cfg.weights()), 175)
  
  
  def test_weights3(self):
    self.cfg.parse('tests/#Data/wosedon_sumo.ini')
    self.assertEquals(self.cfg.weights(), {})
  
  
  def assert_types(self):
    for i in self.cfg.gbuilders():
      self.assertIsInstance(i, GraphBuilderInterface)
      
    for i in self.cfg.mergers():
      self.assertIsInstance(i, MergerInterface)
      
    for i in self.cfg.rerankers():
      self.assertIsInstance(i, RerankInterface)
      
    self.assertIsInstance(self.cfg.wsd_algorithm(), WSDAlgorithmInterface)
  
  
  def test_file1(self):
    self.cfg.parse('tests/#Data/wosedon.ini')
    self.assert_types()
    self.assertEquals(len(self.cfg.gbuilders()), 1)
    self.assertEquals(len(self.cfg.mergers()), 0)
    self.assertEquals(len(self.cfg.rerankers()), 0)
    
    self.assertIsInstance(self.cfg.gbuilders()[0], SynsetGraphBuilder)
    self.assertIsInstance(self.cfg.wsd_algorithm(), LeskAlg)
    
    lfo = self.cfg.algorithm_options().lesk_filter_opts()
    self.assertEquals(lfo.allow_only(), False)
    self.assertEquals(lfo.input_file(), '/home/dekakaruk/Źródła/wosedon/wosedon/poses')
  
  
  def test_file2(self):
    self.cfg.parse('tests/#Data/wosedon_lu.ini')
    self.assert_types()
    self.assertEquals(len(self.cfg.gbuilders()), 2)
    self.assertEquals(len(self.cfg.mergers()), 1)
    self.assertEquals(len(self.cfg.rerankers()), 2)
    
    self.assertIsInstance(self.cfg.gbuilders()[0], SynsetGraphBuilder)
    self.assertIsInstance(self.cfg.gbuilders()[1], LexicalUnitGraphBuilder)
    self.assertIsInstance(self.cfg.mergers()[0], SynsetsLUMerger2)
    self.assertIsInstance(self.cfg.rerankers()[0], LemmaRankingNormalizer)
    self.assertIsInstance(self.cfg.rerankers()[1], LemmaRankingSelecter)
    self.assertIsInstance(self.cfg.wsd_algorithm(), GTStaticPR)
    
    lfo = self.cfg.algorithm_options().lesk_filter_opts()
    self.assertEquals(lfo.allow_only(), False)
    self.assertEquals(lfo.input_file(), None)
  
  
  def test_file3(self):
    self.cfg.parse('tests/#Data/wosedon_sumo.ini')
    self.assert_types()
    self.assertEquals(len(self.cfg.gbuilders()), 2)
    self.assertEquals(len(self.cfg.mergers()), 1)
    self.assertEquals(len(self.cfg.rerankers()), 0)
    
    self.assertIsInstance(self.cfg.gbuilders()[0], SynsetGraphBuilder)
    self.assertIsInstance(self.cfg.gbuilders()[1], SUMOGraphBuilder)
    self.assertIsInstance(self.cfg.mergers()[0], SynsetsSUMOMerger)
    self.assertIsInstance(self.cfg.wsd_algorithm(), GTPersonalizedPRNorm2)
    
    lfo = self.cfg.algorithm_options().lesk_filter_opts()
    self.assertEquals(lfo.allow_only(), True)
    self.assertEquals(lfo.input_file(), '/home/dekakaruk/Źródła/wosedon/wosedon/poses')
    















#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os
from wosedon.basegraph import BaseGraph
from graphbuilderinterface import GraphBuilderInterface

class MSRGraphBuilder(GraphBuilderInterface):
  """!
  Class for building graph from k-best file.

  The built graph is based on the k-best file. K-best file contain words, where 
  for each word there are a list of k most relatedness word to it. To each word 
  in the list of k most relatedness words is also assigned the value of the 
  relatedness measure. K-best file is created based on the corpus and on 
  Measures of Semantic Relatedness.

  The difference between BidirectionalMSRGraphBuilder class and MSRGraphBuilder 
  class is that the BidirectionalMSRGraphBuilder class take into account only 
  symmetric (bidirectional) relations between words.

  Sample format of k-best file:
  \code
  subst:akcyza
    subst:podatek akcyzowy;0.491485
    subst:podatek;0.360689
    subst:agresja;0.235534
  subst:agresja
    subst:przemoc;0.334895
    subst:agresywnosc;0.327364
    subst:akcyza;0.235534
  ...
  \endcode
  """
  def __init__(self, resources, options, str_name = 'MSRGraphBuilder'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of BuildOptions class contain build options from 
                       config file
    @param str_name  - default value same as the class name 
                       "MSRGraphBuilder"
    """
    super(MSRGraphBuilder, self).__init__(resources, options, str_name)

  def build_graph(self):
    """!
    Build graph based on k-best file. Each node has an attribute named "msr" 
    which contain word and POS written as word/POS, e.g. akcyza/2. If word does not 
    have coarse POS, then will be ignored. To each edge is assigned weight, 
    which is the value of measure of semantic relatedness between linked words.

    The graph will be always undirected.

    @return object of BaseGraph class
    """
    if not os.path.exists(self.resources().msr_file()):
      raise IOError(
        "%s file not found!" % \
          self.resources().msr_file())

    g = BaseGraph()
    g.init_graph(drctd = False)

    g.create_node_attribute("msr", "string")
    g.create_edge_attributes([("weight", "double"), ('rel', 'string')])

    parent = None
    child = None
    skip = False
    with open(self.resources().msr_file()) as msrgfile:
      for line in msrgfile:
        if not line.startswith('\t'):
          lemma = line.strip().split(':')[1]
          pos_int = self._convert_to_coarse_pos(line.strip().split(':')[0])
          if not pos_int:
            skip = True
          else:
            skip = False
            parent = lemma + '/' + str(pos_int)
            g.add_node(parent, [("msr", parent)])
        else:
          if skip:
            continue
          lemma = line.strip().split(':')[1].split(';')[0]
          pos_int = self._convert_to_coarse_pos(line.strip().split(':')[0])
          weight = float(line.strip().split(':')[1].split(';')[1])
          if not pos_int:
            continue
          else:
            child = lemma + '/' + str(pos_int)
            g.add_node(child, [("msr", child)])
            if parent != child:
              g.add_edge(parent, child, [("weight", weight)])
    
    if self.options().has_unique_edges():
      g.remove_edge_duplicates()

    return g

  def _convert_to_coarse_pos(self, pl_pos):
    """!
    For a given fine POS return coarse POS in integer type.

    @param pl_pos - fine part of speech
    @return integer coarse part of speech where 1:verb, 2:noun, 3:adverb and 4:adjective
    """
    #! List of tags which can describe nouns.
    noun_pl_pos = ['subst', 'depr', 'ger', 'brev']
    #! List of tags which can describe adjectives.
    adj_pl_pos  = ['adj', 'adja', 'adjp', 'adjc']
    #! List of tags which can describe verbs.
    verb_pl_pos = ['fin', 'bedzie', 'praet', 'impt', \
                   'inf', 'pcon', 'pant', 'imps', \
                   'winien', 'pred', 'pact', 'ppas', 'pred']
    #! List of tags which can describe adverbs.
    adv_pl_pos  = ['adv']

    pos_int = None
    if pl_pos in noun_pl_pos:
      pos_int = 2
    elif pl_pos in adj_pl_pos:
      pos_int = 4
    elif pl_pos in verb_pl_pos:
      pos_int = 1
    elif pl_pos in adv_pl_pos:
      pos_int = 3
    return pos_int



# -*- coding: utf-8 -*-

import unittest, corpus2
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.utils.mergeoptions import MergeOptions
from wosedon.utils.algorithmoptions import AlgorithmOptions
from wosedon.context.document import Document
from wosedon.gbuilders import SynsetGraphBuilder
from wosedon.gbuilders import SUMOGraphBuilder
from wosedon.mergers import SynsetsSUMOMerger
from wosedon.algorithms import *
from wosedon.basegraph import *
        



#FIXME to powinno działać
class TestAllAlgorithms(unittest.TestCase):
  
  def setUp(self):
    # Resources
    res = Resources()
    res.set_tagset('nkjp')
    res.set_plwn_graph_file('tests/#Data/plwnik')
    res.set_sumo_graph_file('tests/#Data/sumo_graph')
    res.set_mapping_sumo_file('tests/#Data/sumo_mapping')
    
    # Graph
    bopts = BuildOptions()
    bopts.set_directed_graph(True)
    bopts.set_unique_edges(False)
    
    mopts = MergeOptions()
    
    gs = SynsetGraphBuilder(res, bopts).build_graph()
    gm = SUMOGraphBuilder(res, bopts).build_graph()
    g = SynsetsSUMOMerger(res, mopts).merge(gs, gm)
    g.create_edge_attribute('weight', 'float', 1.0)
    
    # Context
    cclreader = corpus2.CclRelReader(res.tagset(), 
                                    'tests/#Data/wiki_ccl.xml',
                                    'tests/#Data/wiki_ccl.xml')
    ccldoc = cclreader.read()
    d = Document(res.tagset(), ccldoc, ccldoc.relations())
    
    # Options
    ao = AlgorithmOptions()
    ao.set_ini_nodes('synset')
    
    # Summary
    self.graph = g
    self.resources = res
    self.context = d.next()
    self.options = ao
  
  
  def tearDown(self):
    wsdrank, itr = self.result
    self.assertIsInstance(itr, int)
    self.assertTrue(itr >= 0)
  
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedPR(self):
    alg = GTPersonalizedPR()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedPRNorm(self):
    alg = GTPersonalizedPRNorm()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedPRNorm2(self):
    alg = GTPersonalizedPRNorm2()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedPRNormIt(self):
    alg = GTPersonalizedPRNormIt()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedPRNormReduction(self):
    alg = GTPersonalizedPRNormReduction()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedPRNormTwoStep(self):
    alg = GTPersonalizedPRNormTwoStep()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedW2WPR(self):
    alg = GTPersonalizedW2WPR()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersonalizedW2WPRNorm(self):
    alg = GTPersonalizedW2WPRNorm()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersPRNormItModV(self):
    alg = GTPersPRNormItModV()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersPRNormItModVRankNorm(self):
    alg = GTPersPRNormItModVRankNorm()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTPersPRNormModV(self):
    alg = GTPersPRNormModV()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_GTStaticPR(self):
    alg = GTStaticPR()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking. Moreover, judging from git log, it probably have never worked.")
  def test_GTSUDOKURun2(self):
    alg = GTSUDOKURun2()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_LeskAlg(self):
    self.options.set_lesk_function('Cosine')
    self.resources.set_gloss_file('tests/#Data/glossfile.xml')
    alg = LeskAlg()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)
  
  @unittest.skip("Very mysterious errors, probably the same as in WsdRanking.")
  def test_PaintballWSD(self):
    self.resources.set_impedance_table('tests/#Data/impedance.csv')
    alg = PaintballWSD()
    self.result = alg.run(self.context, self.graph, self.options, self.resources)




























#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='PLWNGraphBuilder',
	version='0.1.0',
	description='PLWN Graph Builder',
	author= "Marlena Orlińska",
	author_email="Marlena.Orlinska@pwr.edu.pl",
        packages=[
            'PLWNGraphBuilder',
            'PLWNGraphBuilder.vertices'
	],
	license='', 
	entry_points={
		'console_scripts': [
			'PLWNGraphBuilder = PLWNGraphBuilder.PLWNGraphBuilder:main'
		]
	}
)



PREC_GEN_DESCRIPTION ='''
Podsumowauje plik *prec i wypisuje ogolna informacje 
o precyzji dla rzeczownikow, czasownikow i srednia z dwoch.
'''

import argparse

def make_parser(desc):
    aparser = argparse.ArgumentParser(description = desc)
    aparser.add_argument('-p', '--precision-filename', 
            dest = 'precision_filename', required = True)
    return aparser

def make_general_stats(prec_filename):
    v_correct = 0
    v_incorrect = 0
    n_correct = 0
    n_incorrect = 0
    
    with open(prec_filename, 'rt') as fin:
        fin.next()
        for line in fin:
            line = line.strip()
            spl_line = line.split(';')
            corr = float(spl_line[5])
            incorr = float(spl_line[4])
            if spl_line[1] == 'n':
                n_correct += corr
                n_incorrect += incorr
            if spl_line[1] == 'v':
                v_correct += corr
                v_incorrect += incorr
    n_sum = (n_correct + n_incorrect)
    n_sum += 0 if n_sum else 1
    
    v_sum = (v_correct + v_incorrect)
    v_sum += 0 if v_sum else 1
    
    n = n_correct / n_sum
    v = v_correct / v_sum
    a = (n_correct + v_correct) / ((n_correct + n_incorrect) + (v_correct + v_incorrect))
    print 'V;', 'Correct;', v_correct, ';Incorrect;', v_incorrect, ';Precision;', v
    print 'N;', 'Correct;', n_correct, ';Incorrect;', n_incorrect, ';Precision;', n
    print '_;', \
            'Correct;', (v_correct+ n_correct), \
            ';Incorrect;', (v_incorrect + n_incorrect), \
            ';Precision;', a
    return [100 * n, 100 * v, 100 * a]

def main(argv = None):
    parser = make_parser(PREC_GEN_DESCRIPTION)
    args = parser.parse_args(argv)

    n, v, d = make_general_stats(args.precision_filename)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='wosedon',
	version='0.1.0',
	description='Word Sense Disambiguation',
	author= "Paweł Kędzia, Marlena Orlińska",
	author_email="Pawel.Kedzia@pwr.edu.pl, Marlena.Orlinska@pwr.edu.pl",
        packages=[
            'wosedon',
            'wosedon.algorithms', 'wosedon.context',
            'wosedon.gbuilders', 'wosedon.mergers', 
            'wosedon.reranks', 'wosedon.utils',
            'wosedon.ranking', 'wosedon.lesk_functions',
            'wosedon.lesk_filters'
	],
	license='LGPL', 
	entry_points={
		'console_scripts': [
			'wosedon = wosedon.run_wosedon:main',
			'posedon = wosedon.run_wosedon_pos:main',
			'wosedon-eval = wosedon.evaluation:main',
			'wosedon-gen-prec = wosedon.prec_general:main'
		]
	}
)



# -*- encoding: utf-8 -*-
__author__ = 'nika'

from productionparseractpcfg import makeActPcfg
import glob
import pickle


############################# CZYTANIE WSZYSTKICH PLIKÓW

all_files = []
for f in glob.iglob("/home/nika/zrobione150326/*/*/*-s.xml"):
    all_files.append(f)

############################# TWORZENIE GRAMATYKI

grammar = makeActPcfg(all_files, 20) ## pliki wejściowe, liczba wątków

############################# ZRZUT DO PLIKU

pickle.dump(grammar, open("pcfg-grammar.pkl", "w"))


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import corpus2
import codecs, os, sys, argparse
import math

import domains as dom
import context.context as ccc
from npsemrel.carrot.db import db
from collections import defaultdict

def make_parser():
  desc = 'Dokonuje oceny UBK-wsd z wzorcowym oznakowaniem kpwr'
  parser = argparse.ArgumentParser(description = desc)
  parser.add_argument('-i', '--index-doc', dest = 'index_doc', required = True)
  parser.add_argument('-r', '--result-filename', dest = 'result_filename', required = True)
  parser.add_argument('-p', '--precision-filename', dest = 'precision_filename', required = True)
  parser.add_argument('-rc', '--recall-filename', dest = 'recall_filename', required = True)
  parser.add_argument('-d', '--db-config', dest = 'db_config', required = True)
  parser.add_argument('-t', '--tagset', dest = 'tagset', default = 'nkjp')
  parser.add_argument('-pr', '--percent-of-rank', dest = 'percent_of_rank', required = False)
  return parser

def make_evaluation(corpus_index_file, result_file, tagset, ann_map, dbconnection, corp_annotations, percent_of_rank):
  full_pos_mask = corpus2.get_attribute_mask(tagset, '')
  ctx  = ccc.WSDContext(tagset, [])

  with codecs.open(result_file, 'wt') as outfile:
    if percent_of_rank:
      outfile.write('POS;POS db;Manual synset ID;Manual synset string;WSD synset ID;WSD synset string;Correctness;WSD ranking of %s%%;Position in rank\n' % (percent_of_rank))
    else:
      outfile.write('POS;POS db;Manual synset ID;Manual synset string;WSD synset ID;WSD synset string;Correctness\n')

    with open(corpus_index_file, 'rt') as corpidx:
      for corpfile in corpidx:
        corpfile = os.path.join(os.path.dirname(corpus_index_file), corpfile).strip()
        print >> sys.stderr, 'Reading:', corpfile
        try:
          cclreader = corpus2.CclRelReader(tagset, corpfile, corpfile)
          document = cclreader.read()
          for paragraph in document.paragraphs(): 
            for sentence in paragraph.sentences(): 
              for token in sentence.tokens():
                md = token.get_metadata()
                if md:
                  WSD_tag = False
                  manual_tag = False
                  pos_id_synset = None

                  ukb_syn_id = md.get_attribute('sense:ukb:syns_id')
                  ukb_syn_unitsstr = md.get_attribute('sense:ukb:unitsstr')
                  ukb_syn_rank = md.get_attribute('sense:ukb:syns_rank')

                  # Sprawdzenie czy w korpusie, przy przy ustawionym
                  # procencie rankingu, ktory ma byc brany
                  # do oceny WSD, znajduje sie tag z rankingiem.
                  if ukb_syn_id and percent_of_rank and not ukb_syn_rank:
                    print >> sys.stderr, 'Brak tagu z rankingiem WSD w korpusie tekstu!'
                    exit(1)
                  
                  # Sprawdzenie czy dla analizowanego tokenu
                  # przypisana jest anotacja WSD.
                  if ukb_syn_id: 
                    WSD_tag = True

                  lemma = str(token.get_preferred_lexeme(tagset).lemma())

                  tag = token.get_preferred_lexeme(tagset).tag()
                  pos_mask = tag.get_masked(full_pos_mask)
                  pos_str = ctx.convert_to_coarse_pos(
                    tagset.tag_to_symbol_string(pos_mask))

                  attribs = md.attributes()
                  for attr_k, attr_v in attribs.iteritems():
                    if attr_k.startswith('sense:wsd_'):
                      if not ann_map.has_key(attr_v):
                        print >> sys.stderr, 'Brak klucza %s w slowniku!' % (attr_v)
                        continue

                      manual_tag = True
                      pos_db = ann_map[attr_v].values()[0][1]
                      man_syn_id = ', '.join([str(synset_id) for synset_id in ann_map[attr_v].iterkeys()])
                      man_syn_str = '|'.join([man_syn_str[0] for man_syn_str in [synset for synset in ann_map[attr_v].itervalues()]])

                      pos_id_synset = '%s;%s;%s' % (
                        pos_str,
                        man_syn_id,
                        man_syn_str)

                      if WSD_tag:
                        if percent_of_rank:
                          ukb_syn_list = [syn.strip() for syn in ukb_syn_rank.split(' ')]
                          number_of_selected_syn = int(math.ceil(float(len(ukb_syn_list)) / 100.0 * float(percent_of_rank)))
                          ukb_syn_list = ukb_syn_list[:number_of_selected_syn]

                          selected_syn = None
                          position_in_rank = None
                          for index, synset in enumerate(ukb_syn_list):
                            synset_id = int(synset.split('/')[0])
                            if ann_map[attr_v].has_key(synset_id):
                              selected_syn = synset_id
                              position_in_rank = index + 1
                              break

                          outfile.write('%s;%s;%s;%s;%s;%s;%d;%s;%d\n' % (
                            pos_str,
                            pos_db,
                            man_syn_id, 
                            man_syn_str,
                            str(selected_syn if selected_syn else ukb_syn_id), 
                            man_syn_str if selected_syn else ukb_syn_unitsstr,
                            1 if selected_syn else 0,
                            ukb_syn_list,
                            position_in_rank if position_in_rank else 0))
                        
                        else:
                          outfile.write('%s;%s;%s;%s;%s;%s;%d\n' % (
                            pos_str,
                            pos_db,
                            man_syn_id,
                            man_syn_str,
                            str(ukb_syn_id), 
                            ukb_syn_unitsstr, 
                            ann_map[attr_v].has_key(int(ukb_syn_id))))

                  if manual_tag:
                    if not corp_annotations.has_key(pos_id_synset):
                      corp_annotations[pos_id_synset] = [0, 0, 0]
                    corp_annotations[pos_id_synset][0] += 1
                    if WSD_tag:
                      corp_annotations[pos_id_synset][1] += 1
                  elif (not manual_tag) and WSD_tag:
                    pos_id_synset = '%s;%s;%s' % (
                      pos_str, 
                      str(ukb_syn_id), 
                      ukb_syn_unitsstr)
                    if not corp_annotations.has_key(pos_id_synset):
                      corp_annotations[pos_id_synset] = [0, 0, 0]
                    corp_annotations[pos_id_synset][2] += 1

        except Exception, e:
          print >> sys.stderr, 'Error: ', e

def precision(result_filename, precision_filename):
  words = {}
  with codecs.open(result_filename, 'rt', 'utf-8') as ifile:
    next(ifile)
    for line in ifile:
      spl_line = line.split(';')
      pos = spl_line[0]
      pos_db = spl_line[1]
      man_syn_id = spl_line[2]
      man_syn_str = spl_line[3]
      word = '%s;%s;%s;%s' % (pos_db, pos, man_syn_id, man_syn_str)
      correctness = int(spl_line[6])
      if not words.has_key(word):
        words[word] = [0, 0]
      words[word][correctness] += 1

  with codecs.open(precision_filename, 'wt', 'utf-8') as ofile:
    ofile.write('POS db;POS;Manual synset ID;Manual synset string;Incorrect;Correct;Precision\n')
    for word, correctness in words.iteritems():
      ofile.write('%s;%d;%d;%f\n' % (word, correctness[0], correctness[1], ((correctness[1] + 0.0) / (0.0 + sum(correctness)))))

def recall(corp_annotations, recall_filename):
  with codecs.open(recall_filename, 'wt') as ofile:
    ofile.write('POS;Manual synset ID;Manual synset string;All manual tags;Manual + WSD tags;Only WSD tags;Recall\n')
    for pos_id_synset, data in corp_annotations.iteritems():
      if data[0] == 0:
        rec = 0
      else:
        rec = (data[1] + 0.0) / (data[0] + 0.0)
      ofile.write('%s;%d;%d;%d;%f\n' % (pos_id_synset, data[0], data[1], data[2], rec))

def make_key(comment):
  return comment.replace('WSD', '').replace(' ', '').replace('#', '-').encode('utf-8')

def make_anotation_dictionary(dbconnection):
  ann_dict = {}

  print >> sys.stderr, 'Getting WSD comments from DB...',
  query = "SELECT SYN.comment, SYN.id, LU.lemma, LU.variant, LU.domain, LU.pos " \
          "FROM lexicalunit LU " \
          "JOIN unitandsynset UAS on (LU.ID = UAS.LEX_ID) " \
          "JOIN synset SYN on (SYN.id = UAS.SYN_ID) " \
          "WHERE SYN.comment LIKE '%wsd%';"
  cursor = dbconnection.cursor()
  cursor.execute(query)

  for row in cursor.fetchall():
    comments = row[0].split(';')[0].split(',')
    synset_id = int(row[1])

    lu_lemma = str(row[2].encode('utf-8')).replace(' ', '_')
    lu_pos = int(row[5])
    lu_variant = int(row[3])
    lu_domain = int(row[4])
    lu_domain_str = dom.DOMAINS[lu_domain] if dom.DOMAINS.has_key(lu_domain) else str(lu_domain)
    manual_synset_str = ('%s.%d(%d:%s) ' % \
                        (lu_lemma, 
                         lu_variant, 
                         lu_domain, 
                         lu_domain_str))

    for comment in comments:
      comment = make_key(comment)
      if ann_dict.has_key(comment):
        if ann_dict[comment].has_key(synset_id):
          ann_dict[comment][synset_id][0] += ' ' + manual_synset_str
          if lu_pos != ann_dict[comment][synset_id][1]:
            print >> sys.stderr, 'Dla synsetu', synset_id, 'wystapily rozne POSy!'
        else:
          ann_dict[comment][synset_id] = [manual_synset_str, lu_pos]
      else:
        ann_dict[comment] = defaultdict(list)
        ann_dict[comment][synset_id] = [manual_synset_str, lu_pos]
  print >> sys.stderr, ' Done!'

  return ann_dict

def main(argv = None):
  parser = make_parser()
  args = parser.parse_args(argv)
  tagset = corpus2.get_named_tagset(args.tagset)

  print >> sys.stderr, 'Connecting to DB...',
  dbcon = db.DB()
  dbconnection = dbcon.connect(args.db_config)
  if not dbconnection:
    print >> sys.stderr, 'Cannot connect to DB!'
    exit(1)
  print >> sys.stderr, ' Done!'

  annotations = make_anotation_dictionary(dbconnection)

  corp_annotations = {}
  make_evaluation(
    args.index_doc, 
    args.result_filename, 
    tagset, 
    annotations, 
    dbconnection,
    corp_annotations,
    args.percent_of_rank)

  precision(args.result_filename, args.precision_filename)
  recall(corp_annotations, args.recall_filename)


if __name__ == '__main__':
  main()



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import os
import operator
from wosedon.basegraph import BaseGraph
from graphbuilderinterface import GraphBuilderInterface

class SynsetGraphBuilder(GraphBuilderInterface):
  """!
  Class for building synset graph.

  The built graph is based on synset graph file. The synset graph is unpickled 
  from file and according to build options set in config file some nodes and edges 
  are filtered out.
  """
  def __init__(self, resources, options, str_name = 'SynsetGraphBuilder'):
    """!
    @param resources - object of Resources class contain resources from config 
                       file
    @param options   - object of BuildOptions class contain build options from 
                       config file
    @param str_name  - default value same as the class name 
                       "SynsetGraphBuilder"
    """
    super(SynsetGraphBuilder, self).__init__(resources, options, str_name)

  def build_graph(self):
    """!
    Build synset graph. 

    At the beginning the synset graph is unpickled from file, which path is set in 
    config file. Next some nodes and edges are filtered out according to build options, 
    which is also set in config file. Each node has an attribute named "synset" which 
    contain synset object. Synset object consist of the following fields: synset_id and 
    lu_set, which contains lexical units objects. Whereas to each edge is assigned relation 
    identifier acquired from PLWN, which occurs between linked synsets.

    The directness of the graph and uniqueness of edges is set in the config file. 
    
    Warning: Nodes (synsets) is filtered out according to part of speech (POS). Each synset contain 
    a set of lexical units. Whereas each lexical unit have assigned one POS. Thus, before node is 
    filtered out the most frequent POS in its lexical units set is checked.

    @return object of BaseGraph class
    """
    syn_graph_file_path = \
      self.resources().plwn_graph_file() + '_syn.xml.gz'

    if not os.path.exists(syn_graph_file_path):
      raise IOError(
        "%s file not found!" % \
          syn_graph_file_path)

    g = BaseGraph()
    g.init_graph(drctd = self.options().is_directed_graph())
    g.unpickle(syn_graph_file_path)
    g.set_directed(self.options().is_directed_graph())

    # Vertices filter
    if self.options().accept_pos() or self.options().accept_lexicon():
      nodes_to_filter_set = set()
      for n in g.all_nodes():
        lu_set = n.synset.lu_set

        if self.options().accept_lexicon():
          lexicon = None if not lu_set else (list(lu_set)[0].lexicon \
            if len(lu_set) == 1 else self._get_most_frequent_lexicon(lu_set))
          if not lexicon in self.options().accept_lexicon():
            nodes_to_filter_set.add(n)

        if self.options().accept_pos():
          pos = None if not lu_set else (list(lu_set)[0].pos \
            if len(lu_set) == 1 else self._get_most_frequent_pos(lu_set))
          if not pos in self.options().accept_pos():
            nodes_to_filter_set.add(n)

      g.nodes_filter(nodes_to_filter_set)

    # Edges
    # Add reversed edges
    
    lst = set()
    for edge in g.all_edges():
      if edge.rel_id in self.options().get_add_reversed_edges():
        lst.add((edge.target(), edge.source(), edge.rel_id))
    for s, t, i in lst:
      ne = g.add_edge(s, t, simply=True)
      ne.rel_id = self.options().get_add_reversed_edges()[i]
    
    # Check edges duplicates
    if self.options().has_unique_edges():
      g.remove_edge_duplicates()
      
    # Edges filter
    if self.options().syn_rel_ids():
      edges_to_filter_set = set()
      for e in g.all_edges():
        if not e.rel_id in self.options().syn_rel_ids():
          edges_to_filter_set.add(e)
      g.edges_filter(edges_to_filter_set)
    
    
    # Type of relation
    g.create_edge_attribute("rel", "string")
    for e in g.all_edges():
      e.rel = 's' + str(e.rel_id)

    return g

  def _get_most_frequent_pos(self, lu_set):
    """!
    Function return the most frequent part of speech in the given lexical units set.

    @param lu_set - set of lexical units objects
    @return the most frequent part of speech in lexical units set
    """
    pos_dict = {}
    for lu in lu_set:
      if lu.pos not in pos_dict:
        pos_dict[lu.pos] = 0
      pos_dict[lu.pos] += 1
    return max(pos_dict.iteritems(), key=operator.itemgetter(1))[0]

  def _get_most_frequent_lexicon(self, lu_set):
    """!
    Function return the most frequent lexicon in the given lexical units set.

    @param lu_set - set of lexical units objects
    @return the most frequent lexicon in lexical units set
    """
    lexicon_dict = {}
    for lu in lu_set:
      if lu.lexicon not in lexicon_dict:
        lexicon_dict[lu.lexicon] = 0
      lexicon_dict[lu.lexicon] += 1
    return max(lexicon_dict.iteritems(), key=operator.itemgetter(1))[0]



# -*- coding: utf-8 -*-

import unittest
from wosedon.basegraph import BaseGraph
from wosedon.basegraph import BaseNode
from wosedon.basegraph import BaseEdge




def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)


class TestBaseGraph_Creation(unittest.TestCase):
  
  def setUp(self):
    self.graph = BaseGraph()
    self.graph.init_graph()
    self.node_a = self.graph.add_node('a')
    self.graph.add_node('b')
    self.graph.add_edge('a', 'b')
    self.graph.add_edge('b', 'a')
    self.graph.add_node('c')
    self.graph.add_node('d')
    self.graph.remove_node('d')
    self.graph.add_node('d')
    self.graph.add_edge('d', 'b')
    self.node_e = self.graph.add_node('e')
    self.edge_ae = self.graph.add_edge('a', 'e')
    self.edge_ee = self.graph.add_edge('e', 'e')
  
  
  def test_vertices(self):
    self.assertEquals(leniter(self.graph.all_nodes()), 5)
    vl = set(self.graph.all_nodes())
    
    for node in self.graph.all_nodes():
      self.assertIsInstance(node, BaseNode)
      self.assertIn(node, vl)  # test for equality and hashes
      self.assertTrue(node == node.copy())
      self.assertFalse(node != node.copy())
  
  
  def test_edges(self):
    self.assertEquals(leniter(self.graph.all_edges()), 5)
    el = set(self.graph.all_edges())
    
    for edge in self.graph.all_edges():
      self.assertIsInstance(edge, BaseEdge)
      self.assertIsInstance(edge.source(), BaseNode)
      self.assertIsInstance(edge.target(), BaseNode)
      self.assertIn(edge, el)  # test for equality and hashes
      self.assertTrue(edge == edge.copy())
      self.assertFalse(edge != edge.copy())
    
    self.assertEquals(self.edge_ee.source(), self.node_e)
    self.assertEquals(self.edge_ee.target(), self.node_e)
    self.assertEquals(self.edge_ae.source(), self.node_a)
    self.assertEquals(self.edge_ae.target(), self.node_e)
    
    self.assertEquals(self.graph.get_edge(self.node_e, self.node_e), self.edge_ee)
    self.assertEquals(self.graph.get_edge(self.node_a, self.node_e), self.edge_ae)
    
    
    


class TestSynsetGB_Changes(unittest.TestCase):
  
  def setUp(self):
    self.graph = BaseGraph()
    self.graph.init_graph()
    
    for i in range(11):
      self.graph.add_node(i)
    
    self.graph.add_edge(1, 3)
    self.graph.add_edge(3, 8)
    self.graph.add_edge(3, 6)
    self.graph.add_edge(3, 6)
    self.graph.add_edge(4, 10)
    self.graph.add_edge(5, 2)
    self.graph.add_edge(5, 10)
    self.graph.add_edge(5, 1)
    self.graph.add_edge(5, 5)
    self.graph.add_edge(7, 1)
    self.graph.add_edge(7, 7)
    self.graph.add_edge(7, 6)
    self.graph.add_edge(8, 5)
    self.graph.add_edge(8, 5)
    self.graph.add_edge(10, 0)
    self.graph.add_edge(0, 10)
    
  
  
  def test_undirected_not_unique(self):
    self.graph.set_directed(False)
    
    self.assertEquals(leniter(self.graph.get_edges_between(3, 6)), 2)
    self.assertEquals(leniter(self.graph.get_edges_between(5, 8)), 2)
    self.assertEquals(leniter(self.graph.get_edges_between(0, 10)), 2)
    self.assertEquals(leniter(self.graph.get_edges_between(5, 1)), 1)
    self.assertEquals(leniter(self.graph.get_edges_between(1, 5)), 1)
    #self.assertEquals(leniter(self.graph.get_edges_between(7, 7)), 1)  #FIXME to nie przechodzi
  
  
  def test_directed_not_unique(self):
    self.graph.set_directed(True)
    
    self.assertEquals(leniter(self.graph.get_edges_between(3, 6)), 2)
    self.assertEquals(leniter(self.graph.get_edges_between(8, 5)), 2)
    self.assertEquals(leniter(self.graph.get_edges_between(0, 10)), 1)
    self.assertEquals(leniter(self.graph.get_edges_between(5, 1)), 1)
    self.assertEquals(leniter(self.graph.get_edges_between(1, 5)), 0)
    self.assertEquals(leniter(self.graph.get_edges_between(7, 7)), 1)
  
  
  def test_undirected_unique(self):
    self.graph.set_directed(False)
    self.graph.remove_edge_duplicates()
    
    for s in self.graph.all_nodes():
      for t in self.graph.all_nodes(): #FIXME usuwanie duplikatów nie obejmuje sytuacji gdy graf jest nieskierowany i krawędź przeciwna tworzy drugą krawędź
        self.assertLessEqual(leniter(self.graph.get_edges_between(s, t)), 2) #1)
  
  
  def test_directed_unique(self):
    self.graph.set_directed(True)
    self.graph.remove_edge_duplicates()
    
    for s in self.graph.all_nodes():
      for t in self.graph.all_nodes():
        self.assertLessEqual(leniter(self.graph.get_edges_between(s, t)), 1)
  
  
  def test_nodes_filter(self):
    self.graph.nodes_filter({1, 5, 8})
    self.assertEquals(leniter(self.graph.all_nodes()), 8)
  
  
  def test_nodes_filter_inv(self):
    self.graph.nodes_filter({1, 5, 8}, inverted=True)
    self.assertEquals(leniter(self.graph.all_nodes()), 3)
  
  
  def test_basenodes_filter(self):
    self.graph.nodes_filter(set(self.graph.all_nodes()))
    self.assertEquals(leniter(self.graph.all_nodes()), 0)
  
  
  def test_edges_filter(self):
    self.graph.edges_filter({self.graph.get_edge(1, 3), self.graph.get_edge(5, 5), self.graph.get_edge(7, 1)})
    self.assertEquals(leniter(self.graph.all_edges()), 13)




class TestBaseGraph_Properties(unittest.TestCase):
  
  def setUp(self):
    self.graph = BaseGraph()
    self.graph.init_graph()
    self.node_a = self.graph.add_node('a')
    self.graph.add_node('b')
    self.graph.add_edge('a', 'b')
    self.graph.add_edge('b', 'a')
    self.graph.create_edge_attribute('weight', 'float')
    self.graph.add_node('c')
    self.graph.add_node('d')
    self.graph.add_edge('d', 'b')
    self.node_e = self.graph.add_node('e')
    self.edge_ae = self.graph.add_edge('a', 'e')
    self.edge_ee = self.graph.add_edge('e', 'e')
    self.graph.create_node_attributes([('colour', 'string'), ('synset', 'object')])
    
  
  def test_properties(self):
    self.node_a.colour = 'green'
    self.assertEquals(self.node_a.colour, 'green')
    
    self.node_e.colour = 'red'
    self.assertEquals(self.node_e.colour, 'red')
    self.assertEquals(self.node_a.colour, 'green')
    
    self.edge_ae.weight = 23
    self.assertEquals(self.edge_ae.weight, 23)
    
  
  def test_ensure_properties(self):
    self.node_a.colour = 'green'
    self.graph.create_node_attributes([('colour', 'string')])
    self.assertEquals(self.node_a.colour, 'green')
    self.graph.create_node_attribute('colour', 'string')
    self.assertEquals(self.node_a.colour, 'green')
    
    self.edge_ae.weight = 23
    self.graph.create_edge_attributes([('weight', 'float')])
    self.assertEquals(self.edge_ae.weight, 23)
    self.graph.create_edge_attribute('weight', 'float')
    self.assertEquals(self.edge_ae.weight, 23)
  
  
  def test_get_node_for_synset_id(self):
    class Synset:
      def __init__(self, syn_id):
        self.synset_id = syn_id
    
    i = 1
    for node in self.graph.all_nodes():
      node.synset = Synset(i)
      i += 1
      
    i = 1
    for node in self.graph.all_nodes():
      self.assertIsInstance(self.graph.get_node_for_synset_id(i), BaseNode)
      i += 1























#encoding: utf8
#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2015  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#
'''
WoSeDon plugin. It can be used into our web services. 

Sample usage:

# ------ CUT HERE
#encoding: utf8
from wosedon import wosedon_plugin 

def madafaka():
  cfg = '/home/pkedzia/repos/wosedon/wosedon/cfg/wosedon_official.ini'
  mdir = '/home/pkedzia/repos/wosedon/wosedon/resources/plug_model_dir'
  wsdlg = wosedon_plugin.WoSeDonPlugin(cfg, mdir)

  fpaths = [
    '/home/pkedzia/test-wosedon/in_1.ccl',
    '/home/pkedzia/test-wosedon/in_2.ccl',
    '/home/pkedzia/test-wosedon/in_3.ccl']

  for inccl in fpaths:
    outccl = inccl + ".wosedon.ccl"
    wsdlg.run_wosedon(inccl, outccl)
# ------ CUT HERE
'''

import logging, sys
import run_wosedon

class WoSeDonPlugin(object):
  def __init__(self, cfg_file_path, model_dir, verbose=True, argv=[]):
    """!
    Constructor makes the model of WoSeDon.

    Reads all required resources, build (or read) model and makes 
    the required objects. Uses the run_wosedon._make_parser method to
    parse available wosedon options.

    @param cfg_file_path path to wosedon config file
    @type cfg_file_path: String
    @param model_dir Path to directory when the built model will be stored.
    @type model_dir: String or None
    @param verbose Is set to true then plugin verbose mode is activated
    @type verbose: Boolean default True
    @param argv additional arguments
    @type argv: list default empty
    """
    self._verbose = verbose
    if verbose:
      logging.debug("Initializing WoSeDon plugin...")

    if verbose:
      logging.debug("Parsing WoSeDon arguments...")
    margv = argv
    margv.extend(['-f', "-"])
    margv.extend(['-c', cfg_file_path])
    margv.extend(['-md', model_dir] if model_dir else [])
    margv.extend(['-V'] if verbose else [])
    p = run_wosedon._make_parser(__doc__)
    self._args = p.parse_args(margv)

    if verbose:
      logging.debug("Reading WoSeDon config file...")
    self._wosedon_cfg = run_wosedon._read_config(self._args.config)

    if verbose:
      logging.debug("Reading WoSeDon models and resources...")
    self._builders, self._mergers, self._wsd_alg, self._g = \
      run_wosedon._parse_cfg_object(self._wosedon_cfg, self._args)

    if verbose:
      logging.debug("WoSeDon was initialized!")

    self._context = None

  @property
  def builders(self):
    """! Getter for builders """
    return self._builders

  @property
  def mergers(self):
    """! Getter for mergers """
    return self._mergers

  @property
  def algorithm(self):
    """! Getter for algorithm"""
    return self._wsd_alg

  @property
  def graph(self):
    """! Getter for graph """
    return self._g
  
  @property
  def config(self):
    """! Getter for read wosedon config """
    return self._wosedon_cfg

  def run_wosedon(self, in_cclfilepath, out_cclfilepath, options=None, 
                  model_result_list=None):
    """!
    Run disambiguation for given intput ccl file path.
    
    Store the result into output file in CCL format.
    
    @param in_cclfilepath
    @type in_cclfilepath: 
    @param out_cclfilepath
    @type out_cclfilepath:
    @param options
    @type options:
    """
    if self._verbose:
      logging.debug('Running WoSeDoN for file: %s', 
                    in_cclfilepath.decode('utf8'))

    self._context = self._wosedon_cfg.context(in_cclfilepath, in_cclfilepath)
    run_wosedon._run_disambiguation(
            self._g, self._context, self._wsd_alg, self._wosedon_cfg, 
            None, self._verbose, None, out_cclfilepath, None, 
            model_result_list)


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2015  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import abc

class RerankInterface(object):
  def __init__(self, str_name):
    self._str_name = str_name

  def __str__(self):
    """String-like representation of reranker"""
    return self._str_name

  @abc.abstractmethod
  def rerank(self, ranking, graph, options):
    """Abstract method. Should be implemented by all rerankers.

    @param ranking - ranking returned from wsdakgorithm.run() method
    @param graph - built graph, object of BaseGraph
    @returns new, reranked ranking with format as below:
    
    [
      (
        token, 
        [
          (<Vertex object with index '6085' at 0x7f91e0a50350>, ranking_value),
          ...
        ]
      ),
      ...
    ]
    """
    raise NotImplementedError("RerankInterface!")



# -*- coding: utf-8 -*-

import unittest, re
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.utils.mergeoptions import MergeOptions
from wosedon.gbuilders import SynsetGraphBuilder
from wosedon.gbuilders import WNDGraphBuilder
from wosedon.mergers import SynsetsWNDMerger2





def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)



@unittest.skip("Mergers even don't know that duplicates should be removed.'")  #FIXME to powinno działać
class TestSynsetsWNDMerger2_DirectnessAndDuplicates(unittest.TestCase):
  
  def setUp(self):
    res = Resources()
    res.set_plwn_graph_file('tests/#Data/plwnik')
    res.set_wnd_graph_file('tests/#Data/wnd_graph')
    res.set_mapping_wnd_file('tests/#Data/wnd_mapping')
    
    bopts = BuildOptions()
    bopts.set_directed_graph(False)
    bopts.set_unique_edges(True)
    
    mopts = MergeOptions()
    
    gs = SynsetGraphBuilder(res, bopts).build_graph()
    gw = WNDGraphBuilder(res, bopts).build_graph()
    self.graph = SynsetsWNDMerger2(res, mopts).merge(gs, gw)
  
  
  def test_undirected_unique(self):
    for s in self.graph.all_nodes():
      for t in self.graph.all_nodes():
        if s != t:
          if leniter(self.graph.get_edges_between(s, t)) > 1:
            self.fail((s, t))
          self.assertLessEqual(leniter(self.graph.get_edges_between(s, t)), 1)
        



class TestSynsetsWNDMerger2(unittest.TestCase):
  
  def setUp(self):
    res = Resources()
    res.set_plwn_graph_file('tests/#Data/plwnik')
    res.set_wnd_graph_file('tests/#Data/wnd_graph')
    res.set_mapping_wnd_file('tests/#Data/wnd_mapping')
    
    bopts = BuildOptions()
    bopts.set_directed_graph(True)
    bopts.set_unique_edges(False)
    
    mopts = MergeOptions()
    
    gs = SynsetGraphBuilder(res, bopts).build_graph()
    gw = WNDGraphBuilder(res, bopts).build_graph()
    self.graph = SynsetsWNDMerger2(res, mopts).merge(gs, gw)
  
  
  def test_edge_names(self):
    for e in self.graph.all_edges():
      self.assertTrue(e.rel in ['isa', 'syn-wnd'] or re.match('s\d+', e.rel))
  
  
  def test_node_types(self):
    for n in self.graph.all_nodes():
      self.assertIsInstance(n.synset.synset_id, int)
      self.assertIsInstance(n.synset.lu_set, set)
  
  
  def test_numbers(self):
    self.assertEquals(leniter(self.graph.all_nodes()), 11)
    self.assertEquals(leniter(self.graph.all_edges()), 32)






























# -*- coding: utf-8 -*-

import unittest
from wosedon.context.sentence import Sentence
import corpus2





def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)




class TestSentence(unittest.TestCase):
  
  def setUp(self):
    tagset = corpus2.get_named_tagset('nkjp')
    cclreader = corpus2.CclRelReader(tagset, 'tests/#Data/wiki_ccl.xml', 'tests/#Data/wiki_ccl.xml')
    ccldoc = cclreader.read()
    self.sentence = Sentence(tagset, ccldoc, ccldoc.relations())
  
  
  def test_basics(self):
    sc = 0
    tc = 0
    
    sen = self.sentence.next()
    while sen:
      sc += 1
      for t in sen.tokens():
        tc += 1
      sen = self.sentence.next()
    
    self.assertEquals(sc, 20)
    self.assertEquals(tc, 411)
    
    self.sentence.reset()
    
    sc = 0
    tc = 0
    
    sen = self.sentence.next()
    while sen:
      sc += 1
      for t in sen.tokens():
        tc += 1
      sen = self.sentence.next()
    
    self.assertEquals(sc, 20)
    self.assertEquals(tc, 411)
  
  
  def test_wsd(self):
    sen = self.sentence.next()
    while sen:
      for t in sen.tokens():
        self.assertIsInstance(sen.get_token_lemma_str(t), str)
        self.assertIn(sen.get_token_coarse_pos(t), ['n', 'v', 'a', None])
      sen = self.sentence.next()






























# -*- coding: utf-8 -*-

import unittest
from wosedon.utils.resources import Resources
from wosedon.utils.buildoptions import BuildOptions
from wosedon.gbuilders.bidirectionalmsrgraphbuilder import BidirectionalMSRGraphBuilder





def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)


class TestBidirectionalMSRGB_Internals(unittest.TestCase):
  
  def setUp(self):
    res = Resources()
    res.set_msr_file('tests/#Data/msr_graph')
    opts = BuildOptions()
    self.gb = BidirectionalMSRGraphBuilder(res, opts)
  
  def test_read_kbests(self):
    self.gb._read_kbests()
    
    self.assertEquals(len(self.gb._kbests_dict), 5)
    
    cnt = 0
    for k in self.gb._kbests_dict:
      for a, b in self.gb._kbests_dict[k]:
        self.assertIsInstance(a, str)
        self.assertIsInstance(b, float)
        cnt += 1
    
    self.assertEquals(cnt, 11)
  
  def test_bidirectional_kbest(self):
    self.gb._read_kbests()
    self.gb._bidirectional_kbest()
    
    self.assertEquals(len(self.gb._kbests_dict), 4)
    
    cnt = 0
    for k in self.gb._kbests_dict:
      for a, b in self.gb._kbests_dict[k]:
        self.assertIsInstance(a, str)
        self.assertIsInstance(b, float)
        cnt += 1
    
    self.assertEquals(cnt, 4)





class TestBidirectionalMSRGB(unittest.TestCase):
  
  def setUp(self):
    self.res = Resources()
    self.res.set_msr_file('tests/#Data/msr_graph')
    self.opts = BuildOptions()
  
  
  def test_edge_names(self):
    g = BidirectionalMSRGraphBuilder(self.res, self.opts).build_graph()
    
    for e in g.all_edges():
      self.assertEquals(e.rel, '')
      self.assertIsInstance(e.weight, float)
  
  
  def test_numbers(self):
    g = BidirectionalMSRGraphBuilder(self.res, self.opts).build_graph()
    
    self.assertEquals(leniter(g.all_nodes()), 4)
    self.assertEquals(leniter(g.all_edges()), 2)
  
  
  def test_unique(self):
    g = BidirectionalMSRGraphBuilder(self.res, self.opts).build_graph()
    
    for s in g.all_nodes():
      for t in g.all_nodes():
        self.assertLessEqual(leniter(g.get_edges_between(s, t)), 1)






























from wosedon.lesk_filters.leskfilterinterface import LeskFilterInterface



class No(LeskFilterInterface):
  """! Forbids everything, probably is useless. """
  
  def __init__(self, tagset, options, str_name='No'):
    super(No, self).__init__(tagset, options, str_name)
  
  def __contains__(self, lexeme):
    return False



#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

# Normalizacja dla lematow w wektorze inicjujacym do <0,1> dla synsetow lematu
# Zmiana wektora personalizacyjnego co iteracje na wynik pr pomnozony przez 
# poczatkowy pr
class GTPersPRNormModV(WSDAlgorithmInterface):
  def __init__(self, str_name = 'GTPersPRNormModV'):
    super(GTPersPRNormModV, self).__init__(str_name)
    self._context_node_dict = None

  def prepare_v(self, wsd_context, graph):
    v = graph.use_graph_tool().new_vertex_property("double")
    for node in self._context_node_dict:
      v[node.use_graph_tool()] = self._multiply_factor * self._context_node_dict[node]
    return v

  def prepare_new_v(self, pers_v, ranking, graph):
    v = graph.use_graph_tool().new_vertex_property("double")
    if not ranking:
      for node in self._context_node_dict:
        v[node.use_graph_tool()] = self._multiply_factor * self._context_node_dict[node]
    else:
      for node in graph.all_nodes():
        v[node.use_graph_tool()] = (pers_v[node.use_graph_tool()] + ranking[node.use_graph_tool()])
    return v

  def _normalize_gt_ranking(self, wsd_context, g, ranking, lemma_on_node_dict):
    # TODO: Moze to warto dodac do wsd_context, metode typu: lemmas_poses()?
    l_pos = set([
      (wsd_context.get_token_lemma_str(t),
       wsd_context.get_token_coarse_pos(t)) for t in wsd_context.tokens()])
    for lemma, pos_str in l_pos:
      s = 0
      for vsyn in lemma_on_node_dict[(lemma, pos_str)]:
        s += ranking[vsyn]
      for vsyn in lemma_on_node_dict[(lemma, pos_str)]:
        ranking[vsyn] /= s
    return ranking
  
  def run(self, wsd_context, graph, options, resources):
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())
    self._set_context_node_dict(lemma_on_node_dict)

    pers_v = self.prepare_v(wsd_context, graph)
    ranking = None
    for i in range(0, options.max_iter()):
      # Prepare the new personalized vector which is multiplication of
      # the ranking from previous loop with personalized page rank given 
      # on the begining of the algorithm
      m_pers_v = self.prepare_new_v(pers_v, ranking, graph)
      (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                   pers = m_pers_v,
                                   prop = ranking,
                                   max_iter = 2,
                                   damping = options.damping_factor(),
                                   ret_iter = True,
                                   weight = graph.use_graph_tool().ep["weight"])
    
    ranking = graph.ungraph_tool(ranking)
    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, ranking)

    return (wsd_rank, ret_iter)

  def _set_context_node_dict(self, lemma_on_node_dict):
    self._context_node_dict = {}
    for nodes in lemma_on_node_dict.itervalues():
      synset_set_size = float(len(nodes))
      for node in nodes:
        if self._context_node_dict.has_key(node):
          self._context_node_dict[node] += 1.0 / synset_set_size
        else:
          self._context_node_dict[node] = 1.0 / synset_set_size



from abc import abstractmethod



class LeskFilterInterface(object):
  """! Interface that shall be implemented by all filters of Lesk. """
  
  def __init__(self, tagset, options, str_name):
    """!
    Standard constructor.
    
    @type tagset: corpus2.Tagset
    @param tagset: tagset
    
    @type options: dict[str, str]
    @param options: dictionary with options read from config file
    
    @type str_name: str
    @param str_name: name to be displayed, it should be name of class
    """
    self._str_name = str_name
    self.tagset = tagset

  def __str__(self):
    return self._str_name
  
  @abstractmethod
  def __contains__(self, lexeme):
    """!
    Check if lemma is allowed
    
    @type lexeme: corpus2.Lexeme
    @param lexeme: lexeme to be checked
    
    @rtype: bool
    @return: True if lexeme is allowed
    """
    raise NotImplementedError("LeskFunctionInterface!")
















from synsetslumerger import SynsetsLUMerger
from synsetslumerger2 import SynsetsLUMerger2
from synsetssumomerger import SynsetsSUMOMerger
from synsetssumomerger2 import SynsetsSUMOMerger2
from synsetswndmerger import SynsetsWNDMerger
from synsetswndmerger2 import SynsetsWNDMerger2
from synsetsmsrmerger import SynsetsMSRMerger
from synsetscclmerger import SynsetsCCLMerger



import corpus2
import codecs, os, sys, argparse
from npsemrel.carrot.db import db
from collections import defaultdict


def make_parser():
  desc = 'Narzedzie generujace statystyke korpusu (liczba znaczen przypadajaca na slowa).'
  parser = argparse.ArgumentParser(description = desc)
  parser.add_argument('-i', '--index-doc', dest = 'index_doc', required = True)
  parser.add_argument('-r', '--result-filename', dest = 'result_filename', required = True)
  parser.add_argument('-d', '--db-config', dest = 'db_config', required = True)
  parser.add_argument('-t', '--tagset', dest = 'tagset', default = 'nkjp')
  parser.add_argument('-S', '--sentence', dest = 'sentence', action = 'store_true', required = False)
  parser.add_argument('-m', '--manual-senses-statistic', dest = 'manual_senses_statistic', action = 'store_true', required = False)
  return parser


def convert_to_num_pos(pl_pos):
  noun_pl_pos = ['subst', 'depr', 'ger']
  adj_pl_pos  = ['adj', 'adja', 'adjp', 'adjc']
  verb_pl_pos = ['fin', 'bedzie', 'praet', 'impt', \
                 'inf', 'pcon', 'pant', 'imps', \
                 'winien', 'pred', 'pact', 'ppas', 'pred']
  pos_int = None
  if pl_pos in noun_pl_pos:
    pos_int = 2
  elif pl_pos in adj_pl_pos:
    pos_int = 4
  elif pl_pos in verb_pl_pos:
    pos_int = 1
  return pos_int

def get_number_of_senses(lemma, pos_int, dbconnection):
  query = 'SELECT LU.lemma, LU.pos, UAS.SYN_ID ' \
          'FROM lexicalunit LU ' \
          'JOIN unitandsynset UAS ON (LU.id = UAS.LEX_ID) ' \
          'WHERE LU.lemma = BINARY "' + lemma + '" and LU.pos = ' + str(pos_int) + ';'
  cursor = dbconnection.cursor()
  cursor.execute(query)
  return len(cursor.fetchall())

def make_statistic(corpus_index_file, result_file, tagset, dbconnection, is_sentence):
  statistic_dict = {}
  max_num_of_syn = 0

  full_pos_mask = corpus2.get_attribute_mask(tagset, '')
  with open(corpus_index_file, 'rt') as corpidx:
    for corpfile in corpidx:
      corpfile_path = os.path.join(os.path.dirname(corpus_index_file), corpfile).strip()
      print >> sys.stderr, 'Reading:', corpfile_path
      try:
        cclreader = corpus2.CclRelReader(tagset, corpfile_path, corpfile_path)
        document = cclreader.read()
        
        statistic_dict[corpfile] = {}
        for paragraph in document.paragraphs(): 
          for sentence in paragraph.sentences():
            statistic_dict[corpfile][sentence.id()] = {}
            for token in sentence.tokens():
              # Lemma
              lemma = str(token.get_preferred_lexeme(tagset).lemma())
              # POS
              tag = token.get_preferred_lexeme(tagset).tag()
              pos_mask = tag.get_masked(full_pos_mask)
              pos_int = convert_to_num_pos(tagset.tag_to_symbol_string(pos_mask))
              if not pos_int:
                print >> sys.stderr, 'Niepoprawny POS dla lematu', lemma, '!'
                continue
              # Number of senses
              num_of_syn = get_number_of_senses(lemma, pos_int, dbconnection)
              if statistic_dict[corpfile][sentence.id()].has_key(num_of_syn):
                statistic_dict[corpfile][sentence.id()][num_of_syn] += 1
              else:
                statistic_dict[corpfile][sentence.id()][num_of_syn] = 1
                if num_of_syn > max_num_of_syn:
                  max_num_of_syn = num_of_syn
      except Exception, e:
        print >> sys.stderr, 'Error: ', e

  with codecs.open(result_file, 'wt') as outfile:
    if is_sentence:
      num_of_syn_list = [0] * (max_num_of_syn + 1)
      num_of_syn_str = ';'.join(str(i) for i in xrange(max_num_of_syn + 1))

      outfile.write('Nazwa dokumentu;Numer zdania;%s\n' % num_of_syn_str)
      for corpfile, sentence_dict in statistic_dict.iteritems():
        for sentence, num_of_syn_dict in sentence_dict.iteritems():
          for num_of_syn, num_of_lemmas in enumerate(num_of_syn_list):
            num_of_syn_list[num_of_syn] = num_of_syn_dict[num_of_syn] if num_of_syn_dict.has_key(num_of_syn) else 0
          num_of_syn_str = ';'.join(str(num_of_lemmas) for num_of_lemmas in num_of_syn_list)
          outfile.write('%s;%s;%s\n' % (corpfile.strip(), sentence, num_of_syn_str))
    else:
      num_of_syn_str = ';'.join(str(i) for i in xrange(max_num_of_syn + 1))
      
      outfile.write('Nazwa dokumentu;%s\n' % num_of_syn_str)
      for corpfile, sentence_dict in statistic_dict.iteritems():
        num_of_syn_list = [0] * (max_num_of_syn + 1)
        for num_of_syn, num_of_lemmas in enumerate(num_of_syn_list):
          for sentence, num_of_syn_dict in sentence_dict.iteritems():
            if num_of_syn_dict.has_key(num_of_syn):
              num_of_syn_list[num_of_syn] += num_of_syn_dict[num_of_syn]
        num_of_syn_str = ';'.join(str(num_of_lemmas) for num_of_lemmas in num_of_syn_list)
        outfile.write('%s;%s\n' % (corpfile.strip(), num_of_syn_str))

def make_manual_senses_statistic(corpus_index_file, result_file, tagset, dbconnection, is_sentence):
  ann_dict = make_anotation_dictionary(dbconnection)

  manual_statistic_dict = {}

  full_pos_mask = corpus2.get_attribute_mask(tagset, '')
  with open(corpus_index_file, 'rt') as corpidx:
    for corpfile in corpidx:
      corpfile_path = os.path.join(os.path.dirname(corpus_index_file), corpfile).strip()
      print >> sys.stderr, 'Reading:', corpfile_path
      try:
        cclreader = corpus2.CclRelReader(tagset, corpfile_path, corpfile_path)
        document = cclreader.read()
        
        manual_statistic_dict[corpfile] = {}
        for paragraph in document.paragraphs(): 
          for sentence in paragraph.sentences():
            manual_statistic_dict[corpfile][sentence.id()] = defaultdict(set)
            for token in sentence.tokens():
              # Manual sense
              md = token.get_metadata()
              if md:
                attribs = md.attributes()
                for attr_k, attr_v in attribs.iteritems():
                  if attr_k.startswith('sense:wsd_'):
                    if not ann_dict.has_key(attr_v):
                      print >> sys.stderr, 'Brak klucza %s w PLWN!' % (attr_v)
                      continue

                    # Lemma
                    lemma = str(token.get_preferred_lexeme(tagset).lemma())

                    # POS
                    tag = token.get_preferred_lexeme(tagset).tag()
                    pos_mask = tag.get_masked(full_pos_mask)
                    pos_int = convert_to_num_pos(tagset.tag_to_symbol_string(pos_mask))
                    if not pos_int:
                      print >> sys.stderr, 'Niepoprawny POS dla lematu', lemma, '!'
                      continue

                    manual_statistic_dict[corpfile][sentence.id()][(lemma, pos_int)].add(attr_v)
      except Exception, e:
        print >> sys.stderr, 'Error: ', e

  statistic_dict = {}
  max_num_of_syn = 0
  for corpfile, sentence_dict in manual_statistic_dict.iteritems():
    statistic_dict[corpfile] = {}
    for sentence, lemma_pos_int_dict in sentence_dict.iteritems():
      statistic_dict[corpfile][sentence] = {}
      for manual_senses_set in lemma_pos_int_dict.itervalues():
        num_of_syn = len(manual_senses_set)
        if statistic_dict[corpfile][sentence].has_key(num_of_syn):
          statistic_dict[corpfile][sentence][num_of_syn] += 1
        else:
          statistic_dict[corpfile][sentence][num_of_syn] = 1
          if num_of_syn > max_num_of_syn:
            max_num_of_syn = num_of_syn

  with codecs.open(result_file, 'wt') as outfile:
    if is_sentence:
      num_of_syn_list = [0] * (max_num_of_syn + 1)
      num_of_syn_str = ';'.join(str(i) for i in xrange(max_num_of_syn + 1))

      outfile.write('Nazwa dokumentu;Numer zdania;%s\n' % num_of_syn_str)
      for corpfile, sentence_dict in statistic_dict.iteritems():
        for sentence, num_of_syn_dict in sentence_dict.iteritems():
          for num_of_syn, num_of_lemmas in enumerate(num_of_syn_list):
            num_of_syn_list[num_of_syn] = num_of_syn_dict[num_of_syn] if num_of_syn_dict.has_key(num_of_syn) else 0
          num_of_syn_str = ';'.join(str(num_of_lemmas) for num_of_lemmas in num_of_syn_list)
          outfile.write('%s;%s;%s\n' % (corpfile.strip(), sentence, num_of_syn_str))
    else:
      num_of_syn_str = ';'.join(str(i) for i in xrange(max_num_of_syn + 1))
      
      outfile.write('Nazwa dokumentu;%s\n' % num_of_syn_str)
      for corpfile, sentence_dict in statistic_dict.iteritems():
        num_of_syn_list = [0] * (max_num_of_syn + 1)
        for num_of_syn, num_of_lemmas in enumerate(num_of_syn_list):
          for sentence, num_of_syn_dict in sentence_dict.iteritems():
            if num_of_syn_dict.has_key(num_of_syn):
              num_of_syn_list[num_of_syn] += num_of_syn_dict[num_of_syn]
        num_of_syn_str = ';'.join(str(num_of_lemmas) for num_of_lemmas in num_of_syn_list)
        outfile.write('%s;%s\n' % (corpfile.strip(), num_of_syn_str))

def make_key(comment):
  return comment.replace('WSD', '').replace(' ', '').replace('#', '-').encode('utf-8')

def make_anotation_dictionary(dbconnection):
  ann_dict = defaultdict(set)

  print >> sys.stderr, 'Getting WSD comments from DB...',
  query = "SELECT SYN.comment, SYN.id " \
          "FROM synset SYN " \
          "WHERE SYN.comment LIKE '%wsd%';"
  cursor = dbconnection.cursor()
  cursor.execute(query)

  for row in cursor.fetchall():
    comments = row[0].split(';')[0].split(',')
    synset_id = int(row[1])
    for comment in comments:
      comment = make_key(comment)
      ann_dict[comment].add(synset_id)
  print >> sys.stderr, ' Done!'

  return ann_dict


def main(argv = None):
  parser = make_parser()
  args = parser.parse_args(argv)
  tagset = corpus2.get_named_tagset(args.tagset)

  print >> sys.stderr, 'Connecting to DB...',
  dbcon = db.DB()
  dbconnection = dbcon.connect(args.db_config)
  if not dbconnection:
    print >> sys.stderr, 'Cannot connect to DB!'
    exit(1)
  print >> sys.stderr, ' Done!'

  if args.manual_senses_statistic:
    make_manual_senses_statistic(args.index_doc, args.result_filename, tagset, dbconnection, args.sentence)
  else:
    make_statistic(args.index_doc, args.result_filename, tagset, dbconnection, args.sentence)


if __name__ == '__main__':
  main()


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

import abc

class GraphBuilderInterface(object):
  def __init__(self, resources, options, str_name):
    self._resources = resources
    self._build_options = options
    self._str_name = str_name

  def __str__(self):
    return self._str_name

  def resources(self):
    return self._resources

  def options(self):
    return self._build_options

  @abc.abstractmethod
  def build_graph(self):
    raise NotImplementedError("GraphBuilderInterface!")



#! /usr/bin/python
# *-* coding: utf-8 *-*

from __future__ import print_function
from concrafteusz import Concraft

import codecs
import argparse
import glob
import morfeusz2
import subprocess

parser = argparse.ArgumentParser()
parser.add_argument('files', help="nazwa pliku do analizy (plain text)", nargs='*')
parser.add_argument('-c', '--conll', action='store_true', default=False, help='Wyniki w formacie CoNLL dla parsera zależnościowego.')
args = parser.parse_args()
make_conll = parser.parse_args().conll

#concraftpath = './concraft-dag2-pl'

morfeusz = morfeusz2.Morfeusz(
    dict_name='sgjp',
    expand_tags=True,
    generate=False,
)

tager = Concraft(
    model_name = 'concraft-pl-model-180317.gz',
    model_path = './',
    port = 3005,
    core = 5
)


def delrozp(lemat):
    if lemat != ':':
        lemat = lemat.split(':')[0]
    return(lemat)

if make_conll:
    COMPs = ('przeto', u'jakoż', u'toteż', 'tedy', 'to', 'ergo', 'czym', 'jakokolwiek', u'więc', 'zatem')

    d = {
        'adjnum' : 'adj',
        'advnum' : 'adv',
        'part' : 'qub',
        'fut' : 'bedzie',
        'plusq' : 'praet',
        'frag' : 'burk'
    }

    def conv(tag):
        tag = tag.split(':')
        if tag[0] == 'adjp':
            tag = ['adjp',]
        elif tag[0] == 'ppron12':
            del tag[3] 
        elif tag[0] in d:
            tag[0] = d[tag[0]]
        tag = ':'.join(tag)
        return(tag)

    sent = []

try:
    for plainf in args.files:
        if make_conll:
            taggedf = plainf.replace('.', '_') + '_tagged.conll'
        else:
            taggedf = plainf.replace('.', '_') + '_tagged.dag'
        with codecs.open(plainf, mode='r', encoding='utf-8') as pf:
            with codecs.open(taggedf, mode='w', encoding='utf-8') as tf:
                print('tagging file: ' + str(plainf))
                for line in pf:
                    if line != '\n':
                        analysis = morfeusz.analyse(line)
                        tagged = tager.tag(analysis)
                        for item in tagged:
                            num1, num2, (forma, lemat, tag, posp, kwal), prob, eos, disamb = item
                            lemat = delrozp(lemat)
                            if make_conll:
                                if tag == 'conj' and lemat in COMPs:
                                    tag = 'comp'
                                if disamb == 'disamb':
                                    if eos != 'eos':
                                        sent.append((forma, lemat, conv(tag)))
                                    else:
                                        sent.append((forma, lemat, tag))
                                        for count, item in enumerate(sent, 1):
                                            ctag = item[2].split(':')[0]
                                            feat = '|'.join(item[2].split(':')[1:]) if len(item[2].split(':')) > 1 else '_'
                                            tupla = (str(count), item[0], item[1], ctag, ctag, feat, '_', '_', '_', '_')
                                            print('\t'.join(tupla),file=tf)
                                        print('', file=tf)
                                        sent = []
                            else:
                                print('\t'.join((str(num1), str(num2), forma, lemat, tag, ','.join(posp), ','.join(kwal), str(prob), eos if eos else '', disamb if disamb else '')), file=tf)
finally:
    tager.done()



from setuptools import setup
import setuptools

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name='eniam',
    version='0.17',
    license='MIT',
    author='styczynski',
    author_email='piotr@styczynski.in',
    url='http://styczynski.in',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    scripts=['bin/eniam-cli'],
    include_package_data=True,
    package_data={},
    install_requires=[
        'opencv-python==4.2.0.32',
        'camelot-py==0.7.2',
        'wand==0.5.8',
    ],
    description="Cross-platform python wrapper around ENIAM (http://eniam.nlp.ipipan.waw.pl/)",
)


#
# <one line to give the library's name and an idea of what it does.>
# Copyright (C) 2014  <copyright holder> <email>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
#

from graph_tool.centrality import pagerank
from wosedon.algorithms.wsdalgorithminterface import WSDAlgorithmInterface
from wosedon.ranking.wsd_ranking import WSDRanking

class GTPersonalizedPR(WSDAlgorithmInterface):
  """!
  Personalized Page Rank algorithm based on the set of synsets which
  correspond to lemmas from the analyzed context. All synsets which belong
  to lemmas from the analyzed context have the same value in personalized
  vector "v", which is equal to 1/N, where N is the number of these synsets.
  Remaining synsets which do not belong to lemmas from the analyzed context
  have value equal to 0 in personalized vector "v".
  """

  def __init__(self, str_name = 'GTPersonalizedPR'):
    """!
    @param str_name - default value same as the class name "GTPersonalizedPR"
    """
    super(GTPersonalizedPR, self).__init__(str_name)
    self._context_node_set = None
  
  def prepare_v(self, wsd_context, graph):
    """!
    Create personalized vector "v".

    All synsets which belong to lemmas from the analyzed context have value
    equal to 1/N, where N is the number of these synsets, in personalized vector
    "v".

    Remaining synsets which do not belong to lemmas from the analyzed context
    have value equal to 0 in personalized vector "v".

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @return PropertyMap (see: http://graph-tool.skewed.de/static/doc/graph_tool.html?highlight=new_vertex_property#graph_tool.Graph.new_vertex_property) of personalized vector "v"
    """
    v = graph.use_graph_tool().new_vertex_property("double")
    for node in self._context_node_set:
      v[node.use_graph_tool()] = self._multiply_factor * \
        (1.0 / float(len(self._context_node_set)))
    return v
  
  def run(self, wsd_context, graph, options, resources):
    """!
    Disambiguate analyzed context.

    @param wsd_context - object of WSDContext class
    @param graph - object of BaseGraph class
    @param options - object of AlgorithmOptions class
    @param resources - object of Resources class
    @return tuple of object of WSDRanking class and number of algorithm iterations
    """
    wsd_rank = WSDRanking()
    (lemma_on_node_dict, lemma_on_only_synset_node_dict) = \
      wsd_rank.get_lemma_on_node_dict(wsd_context, graph, options.ini_nodes())
    self._set_context_node_set(lemma_on_node_dict)

    pers_v = self.prepare_v(wsd_context, graph)
    (ranking, ret_iter) = pagerank(graph.use_graph_tool(), 
                                   pers = pers_v, 
                                   max_iter = 2 * options.max_iter(),
                                   damping = options.damping_factor(),
                                   ret_iter = True,
                                   weight = graph.use_graph_tool().ep["weight"])
    
    ranking = graph.ungraph_tool(ranking)
    for (lemma, pos_str) in lemma_on_only_synset_node_dict.iterkeys():
      wsd_rank.set_ranking_for_lemma(lemma, pos_str, ranking)

    return (wsd_rank, ret_iter)

  def _set_context_node_set(self, lemma_on_node_dict):
    """!
    Create a set of synsets (precisely nodes from the graph) which belong
    to lemmas from the analyzed context.

    @param lemma_on_node_dict - dictionary where key is (lemma, pos_str) and
                                value is set of synsets (nodes from the graph)
                                belong to this lemma
    """
    self._context_node_set = set()
    for nodes in lemma_on_node_dict.itervalues():
      for node in nodes:
        self._context_node_set.add(node)



# -*- coding: utf-8 -*-

import unittest
from wosedon.lesk_filters import Words
from wosedon.utils.leskfilteroptions import LeskFilterOptions

import corpus2
from wosedon.context.document import Document




def leniter(i):
  """ Lenght of any iterable """
  return sum(1 for e in i)


class TestLeskFilterWords(unittest.TestCase):

  def setUp(self):
    self.tagset = corpus2.get_named_tagset('nkjp')
    
    cclreader = corpus2.CclRelReader(self.tagset, 'tests/#Data/wiki_ccl.xml', 'tests/#Data/wiki_ccl.xml')
    ccldoc = cclreader.read()
    self.document = Document(self.tagset, ccldoc, ccldoc.relations())


  def test_it(self):
    opts = LeskFilterOptions()
    opts.set_input_file('tests/#Data/leskfilters/words')
    self.filter = Words(self.tagset, opts)
    
    doc = self.document.next()
    while doc:
      for t in doc.tokens():
        lexeme = t.get_preferred_lexeme(self.tagset)
        lemma = str(lexeme.lemma())
        if lemma in ['z', 'kot', 'robić']:
          self.assertNotIn(lexeme, self.filter)
        else:
          self.assertIn(lexeme, self.filter)
          
      doc = self.document.next()

  
  def test_reversed(self):
    opts = LeskFilterOptions()
    opts.set_input_file('tests/#Data/leskfilters/words')
    opts.set_allow_only(True)
    self.filter = Words(self.tagset, opts)
    
    doc = self.document.next()
    while doc:
      for t in doc.tokens():
        lexeme = t.get_preferred_lexeme(self.tagset)
        lemma = str(lexeme.lemma())
        if lemma in ['z', 'kot', 'robić']:
          self.assertIn(lexeme, self.filter)
        else:
          self.assertNotIn(lexeme, self.filter)
          
      doc = self.document.next()






# -*- encoding: utf-8 -*-
__author__ = 'nika'

import random, time
random.seed(time.time())

class Node(object):
    def __init__(self, _args):
        # nid="11" from="3" to="5" subtrees="1" chosen="false"
        self.nid = _args["nid"]
        self.fro = int(_args["from"])
        self.to = int(_args["to"])
        self.subtress = int(_args["subtrees"])
        if "chosen" in _args and _args["chosen"] == "false":
            self.chosen = False
        else:
            self.chosen = True
        self.terminal = False
        self.nonterminal = False
        self.arguments = {}
        self.category = u'terminal'
        self.children = []
        self.parents = {}

    def __unicode__(self):
        return "NID: "+unicode(self.nid)+" "+unicode(self.arguments)

    def getFromToCat(self):
        return unicode(self.getFrom()) + '@' + unicode(self.getTo()) + '@' +unicode(self.getCategory())

    def getArgDict(self, prefix):
        d = {}
        d[prefix+'@'+unicode(self.category)] = 1
        #print self.nid, self.category
        for key in self.arguments.keys():
            if key != "rekcja" and key != "poz":# and not(key == 'ink' and self.category != 'zdanie'):
                d[prefix+'@'+key+'@'+self.arguments[key]] = 1
        return d

    def make_exp_data(self, leaves,dom):
        d = {}
        for ii in range(len(leaves)):
            try:
                d['1gram@base@'+ leaves[ii]["base"]]  = 1
            except KeyError:
                pass
            for jj in leaves[ii]["tag"].split(':'):
                d['1gram@tag@'+jj] = 1
            #if ii < len(leaves)-1:
            #    #d['2gram@tag@' + leaves[ii]["tag"].split(':')[0] + '@' +leaves[ii+1]["tag"].split(':')[0]] = 1
            #    try:
            #        d['2gram@base@' + dom[leaves[ii]["base"]] + '@' + dom[leaves[ii+1]["base"]]] = 1
            #    except KeyError:
            #        pass
            
            #if ii < len(leaves)-2:
            #    d['3gram@tag@' + leaves[ii]["tag"].split(':')[0] + '@' +leaves[ii+1]["tag"].split(':')[0] + '@' + leaves[ii+2]["tag"].split(':')[0]] = 1
        return d

    def isTerminal(self):
        return self.terminal

    def isChosen(self):
        return self.chosen

    def addChildren(self, child):
        self.children.append(child)

    def addArgument(self, arg, value):
        self.arguments[arg] = value

    def setTerminal(self):
        self.terminal = True

    def setNonterminal(self):
        self.nonterminal = True

    def setCategory(self, _cat):
        self.category = _cat

    def getRandomChildren(self):
        return self.children[random.randint(0,len(self.children)-1)]

    def getAllChildren(self):
        #print self.children
        return self.children

    def getChildrenForActPcfg(self):
        ch = [(self.children['centre'], u'true')]
        ch += map(lambda x: (x,u'false'), self.children['productions'])
        return ch

    def getRawChildren(self):
        ch = [(self.children['centre'], u'true')]
        ch += map(lambda x: (x,u'false'), self.children['productions'])
        return ch

    def getCategory(self):
        cat = unicode(self.category)
        if self.category == "fw":
            cat += '@' + unicode(self.arguments['tfw'])
        if "przypadek" in self.arguments.keys():
            pass#cat += '@' + unicode(self.arguments["przypadek"])
        if "rodzaj" in self.arguments.keys():
            pass#cat += '@' + unicode(self.arguments["rodzaj"])
        if "liczba" in self.arguments.keys():
            pass#cat += '@' + unicode(self.arguments["liczba"])
        if "osoba" in self.arguments.keys():
            pass#cat += '@' + unicode(self.arguments["osoba"])
        return cat

    def getRodzaj(self):
        if 'rodzaj' in self.arguments.keys():
            return self.arguments['rodzaj']
        else:
            return None

    def getLiczba(self):
        if 'liczba' in self.arguments.keys():
            return self.arguments['liczba']
        else:
            return None

    def getOsoba(self):
        if 'osoba' in self.arguments.keys():
            return self.arguments['osoba']
        else:
            return None


    def getExtCategory(self):
        #if self.category == "fw":
        #    return unicode(self.category) + '@' + unicode(self.arguments['tfw']) + '@' + unicode(self.getRodzaj()) + '@' + unicode(self.getLiczba()) + '@' +unicode(self.getOsoba())
        return unicode(self.getCategory())# + '@' + unicode(self.getRodzaj()) + '@' + unicode(self.getLiczba()) + '@' +unicode(self.getOsoba())

    def getNode(self):
        return self.getCategory(), self.children

    def getArguments(self):
        return self.arguments

    def getID(self):
        return self.nid

    def getFrom(self):
        return self.fro

    def getTo(self):
        return self.to

    def equals(self, node):
        equal = True
        for arg in self.arguments.keys():
            if arg in node.arguments.keys():
                if not self.arguments[arg] == node.arguments[arg]:
                    equal = False
                    break
            else:
                equal = False
                break
        return equal and self.equals_from_to_cat(node)

    def equals_from_to_cat(self, node):
        return self.getFrom() == node.getFrom() and self.getTo() == node.getTo() and self.getCategory() == node.getCategory()

    def equals_from_to(self, node):
        return self.getFrom() == node.getFrom() and self.getTo() == node.getTo()

    def overlaps(self, node):
        #print self.getFrom(), self.getTo(), node.getFrom(), node.getTo(), self.getFrom() <= node.getTo() and self.getTo() >= node.getFrom()
        return self.getFrom() <= node.getTo() and self.getTo() >= node.getFrom()





